## Copyright (C) 2000 - 2006  SWsoft
## All rights reserved.
## Abstract base class for the engine of payment methods

package HSPC::MT::PM::PluginEngine::BankAccount;

use strict;
use SWsoft::protect;

use HSPC::Logger qw(sw_atrace);  ## sw_log_warn sw_log_ok sw_log sw_die);

use Error qw(:try);
use HSPC::Exception::Plugin::GeneralError;

use HSPC::MT::Billing::Constants qw(:DEFAULT);
use HSPC::MT::PP::Constants;
use HSPC::MT::PM::Tools;

## Inheritance
use HSPC::MT::PM::PluginEngine;

use vars qw{@ISA};
@ISA = qw{HSPC::MT::PM::PluginEngine};

sub apply_to_doc {
	my $self = shift;
	my %h = (@_);

	my $trace = sw_atrace();

	$self->SUPER::apply_to_doc(%h);

	my $config = HSPC::MT::PM::Tools->get_vendor_config(vendor_id => $self->vendor_id);

	if ($config && $config->bt_apply_pm_action_type != SW_PP_BT_APPLY_PM_ACTION_NO) {
		my $ar_doc = $h{ar_doc};
		my $ar_doc_rb = HSPC::MT::BillingFactory->find_ar_doc_rb( id => $ar_doc->id() );

		if ($ar_doc_rb && $ar_doc_rb->method_id() == $self->id()) {
			if (
				$ar_doc->doc_type eq &DT_OR &&
				$ar_doc->doc_status eq &DS_OPEN &&
				$ar_doc->doc_balance > 0
			) {
				$ar_doc = HSPC::MT::Billing->process_order_immediately(
					order_id       => $ar_doc->id(),
					async          => &SW_BM_PROVISIONING_ASYNC,
					sec_env        => $h{sec_env},
					status_comment => 'Order completed by Invoice while applying payment method to doc',
				);
				sw_die('Didn`t get invoice object from process_order_immediately')
					unless ($ar_doc);
			}

			##Add document to batch
			if ($config->bt_apply_pm_action_type == SW_PP_BT_APPLY_PM_ACTION_ADDBATCH) {
				my $plugins = HSPC::MT::PM::Tools->get_plugins_supported_by_paytype(
					type      => $self->type,
					vendor_id => $self->vendor_id,
				);
				if ($plugins && scalar(@$plugins)) {
					my $batch_id = HSPC::MT::Billing::BatchTools->find_open_batch_id(
						vendor_id => $self->vendor_id,
						plugin_id => $plugins->[0],
					);
			
					unless ($batch_id) {
						$batch_id = HSPC::MT::Billing::BatchTools->make_new_batch(
							vendor_id => $self->vendor_id,
							plugin_id => $plugins->[0],
						);
					}

					sw_die("Can't find or create batch") unless $batch_id;

					HSPC::MT::Billing::BatchTools->add_documents_to_batch(
						batch_id => $batch_id,
						doc_ids  => [$ar_doc->id],
					);
				}
			}
		}
	}

	$trace->add("ok");
	return undef;
}

1;
