## Copyright (C) 2006  SWsoft
## All rights reserved.
##
## This file contains Web interface elements
## of class Plugin
## (note: purpose - plugin operation in PCC/RCC->Commerce Director->Payment Plugins screen)
package HSPC::PluginConfig::PP;

use SWsoft::protect;
use SWsoft::HSPC::License;

use strict;
use HSPC::PluginConfig::Abstract;
use HSPC::WebSystemLib;
use HSPC::Logger qw(sw_die sw_log sw_log_warn);
use HSPC::Localization qw(:DEFAULT string current_lang);
use HSPC::MT::GM::Constants qw(:card_pr_type);
use HSPC::WebDB qw(atrans);

use HSPC::MT::Core::Constants qw(SW_HSP);
use HSPC::MT::CCP::Constants qw(:trans_types);
use HSPC::MT::PluginTools qw(set_request_data get_request_data);
use HSPC::WebSystemLib qw(sw_web_parse_template);

use HSPC::PluginWizard::PP;
use HSPC::MT::PP;
use Error qw(:try);

use vars qw(@ISA);
@ISA = qw(HSPC::PluginConfig::Abstract);

sub setup {
	my ($class) = @_;
	my %h = (@_);
	
	LCHECK_CC;

	my $self = $class->new();

	my $page = HSPC::WebPage->new();

	$self->{page} = $page;

	my $sub_type = sw_uriparam('plugin_type') || sw_argparam('plugin_type') ||
		substr($ENV{COMPREP}->node->{alias}, length('payment_plugins_'));
	
	$self->{plugin_sub_type} = uc($sub_type);

	my $id  = sw_argparam("id")  || sw_uriparam('id');
	my $act = sw_argparam("act") || sw_uriparam('act');

	$self->{obj} = HSPC::MT::PluginConfig->find_plugin(
		id                      => $id,
		vendor_id               => $ENV{security_obj}->account_no(),
		force_activity          => 1,
		force_template_existing => 1,
	) if $id;

	if ($self->{obj}) {
		set_request_data('plugin_vendor_id', $self->{obj}->{plugin}->vendor_id() );
		set_request_data('plugin_config_id', $self->{obj}->{plugin}->id() );
		set_request_data('plugin_template_id', $self->{obj}->{plugin}->template_id() );
	}

	if ($act eq 'new') {
		return HSPC::PluginWizard::PP->edit_teaser( page => $page, start => 1, sub_type => $sub_type );
	} elsif ($act eq 'wizard') {
		return HSPC::PluginWizard::PP->edit_teaser( page => $page );
	} elsif ($act eq  'view') {
		$self->view();
	} elsif ($act eq 'edit') {
		$self->edit();
	} elsif ($act eq 'update') {
		$self->update();
	} else {
		if ($act eq 'move') {
			HSPC::MT::PP->change_plugins_order(
				move_up      => sw_argparam('btn_up') ? 1 : 0,
				plugin_type  => $self->{plugin_sub_type},
				plugins4move => sw_argparam( 'id', 'list' ) || [],
				vendor_id    => $ENV{security_obj}->account_no,
			);
		}
		$self->list(page => $page);
	}
	return  {STATUS=>($page->{status} || 'OK'),CONTENT=>$page->{edit_out}};
}

sub list{
	my $self = shift;

	my $page = $self->{page};
	my $is_active = &sw_param('is_active');
	
	$page->title(pages => [], refresh_url => $ENV{COMPREP}->link(sw_uriparams()));
	
	my $vendor_id = $ENV{security_obj}->account_no;
	
	$page->{res_itemstitle_id} = 'cc_plugins_uc';
	$page->{res_itemtitle_id}  = 'cc_plugin_uc';
	
	my $templates = HSPC::MT::PluginConfig::PP->find_templates(
				plugin_type => "PP",
				vendor_id   => $vendor_id,
				sub_type    => $self->{plugin_sub_type},
			);

	if (scalar @{$templates} ) {
		$page->actions_bar(btns=>[{title_id=>'new_plugin_uc',
					full_url => $ENV{COMPREP}->link(alias => 'payment_plugins_' . 
						lc($self->{plugin_sub_type}) . '_new', {act => 'new', plugin_type => $self->{plugin_sub_type}} )}]);
	}

	my $available_plugins_arrayref = HSPC::MT::PP->get_pp_plugin_info(
		vendor_id               => $vendor_id,
		sub_type                => $self->{plugin_sub_type},
		force_template_existing => 1,
	);

	my @available_plugins = map { $_->{id} } @$available_plugins_arrayref;
	
	## columns to be shown
	my @cols = (
		{
			title_id  => 'id',
			code      => 'id',
			type      => 'id',
			hide      => 1,
			for_where => 1,
			value     => \@available_plugins,
			for_check => scalar @available_plugins > 1 ? 1 : 0
		},
		{
			title_id => 'sort_order',
			code     => 'sort_order',
			hide     => 1,
			sort_def => 1,
		},
		{
			title_id   => 'name',
			code       => 'name',
			type       => 'q',
			for_search => 1,
			not_escape => 1,
		},
		{ 
			title_id => 'active',
			code     => 'is_active',
			type     => 'bool',
		},
	);

	my $checklist_buttons = [
		{ name => 'btn_up',   icon => 'icon_export', title_id => 'move_up_uc' },
		{ name => 'btn_down', icon => 'icon_import', title_id => 'move_down_uc' }
	];
	
	if (scalar @available_plugins) {
		my $request = $page->prepare_browse( cols=> \@cols );

		HSPC::MT::PluginConfig::PP->load_data(request => $request);

		$request->sort();
		$page->search_bar();

		my $checklist_url = $ENV{COMPREP}->link(
			alias => 'payment_plugins_' . lc($self->{plugin_sub_type}),
			[
				act => 'move'
			]
		);

		## draw plugins list
	    $page->browse(
			browse_info        => $request,
			item_url           => '/act,view',
			checklist_full_url => $checklist_url,
			checklist_buttons  => $checklist_buttons,
			##hidden_values    => {plugin_type => 'credit_card'}
			);
	}
	else {
		$page->search_bar();
		$page->add_status(string_id =>'no_pp_plugin_uc');
		$page->statuses();
	}

    return;
}

sub view {
	my $self = shift;
	
	my $page = $self->{page} or sw_die('page undefined');
	my $obj  = $self->{obj};
	
	## check plugin is presence and defined
	unless ( defined($obj) && $obj->{plugin}->id ) {
		$page->add_error (
			string => string('plugin_unavail') . ' ' . string('try_again')
		);
		return $self->list();
	}

	my $mt_template = HSPC::MT::PluginConfig->find_mt_template(
		template_id => $obj->{plugin}->template_id(),
		plugin_type => $obj->{plugin}->plugin_type,
	);

	if ( !$mt_template ) {
		$page->{common_url} =
		  $ENV{COMPREP}->link( alias => $ENV{COMPREP}->node->{alias}, );
		return $page->redirect(
			'', 1, '',
			string(
				'pc_template_not_installed_uc',
				templ_id => $obj->{plugin}->template_id()
			)
		);
	}

	$ENV{COMPREP}->node(edit => 1)->{title_hash}->{title} =
		$obj->{plugin}->name . " ". string('plugin');

	$page->title(
		pages       => [],
		refresh_url => $ENV{COMPREP}->link( sw_uriparams() )
	);


	$page->view_info_header();

	$page->view_info_folder(title_id => 'gen_settings');

	$page->view_info_text(
		title_id   => 'plugin_name_uc',
		value      => $obj->{plugin}->name(),
	);	
	
	$page->view_info_text(
		title_id   => 'plugin_template_uc',
		value      => $mt_template->get_title(),
	);
	
	$page->view_info_text(
		title_id => 'active',
		value    => $obj->{plugin}->is_active,
		type     => 'bool',
	);
	
	if ( $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_PREAUTH) 
		&& $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_SALE) )
	{
		$page->view_info_text(
			title_id => 'preauth_enabled',
			type     => 'bool',
			value    => $obj->{plugin}->is_preauth_enabled
		);
	}
	
	if ( $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_PREAUTH) 
		&& $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_CAPTURE)
		&& ( !$obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_SALE)
			|| $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_SALE )
				&& $obj->{plugin}->is_preauth_enabled
			)
		)
	{
		$page->view_info_text(
			title_id => 'start_billing_trans_uc',
			value    => string (
							&SW_CCP_TRANS_TYPE_TXT->{
									$obj->{plugin}->start_billing_trans
								} 
						)
		);
		
	}

	if ( scalar @{$obj->{plugin}->get_supported_payment_method_types} > 1 ) {
		$page->view_info_text(title_id=>'supported_cardtypes', as_plain=>0,
								value=>join('&nbsp;', map { "<img_$_->{image_id}>" }
									@{$obj->{plugin}->paymethod_types}));
	}

	$page->view_info_folder(title_id => 'plugin_setting_uc');
	my $pr_template = $obj->{plugin}->find_presentation_template();

	my $is_provider = $ENV{security_obj}->account_type == &SW_HSP ? 1 : 0;

	my $content;
	eval {
		$content = $pr_template->view_form(
						config      => $obj->{plugin}->config_data(),
						is_provider => $is_provider
					);
	};

	if ($@) {
		sw_log_warn("Error load presentation form: " . $@);
	}

	$page->edit_append( content => $content );
	
	$page->view_info_folder(title_id => 'misc_cfg');

	$page->view_info_text(
		title_id => 'quick_descr_of_payment',
		value    => $obj->{plugin}->banner_text
	);

	$page->view_info_footer();
	
	my $is_active = $obj->{plugin}->is_active;

	my @buttons;
	if (!$is_active) {
		push @buttons, [ 'mark_active', string('mark_active'), 'SWIButton', 'submit'];
	}

	## draw "Deactivate" button if plugin currently active
	push @buttons, [ 'deactivate_plugin', string('deactivate'), 'SWIButton', 'submit'] if ($is_active);
	
	$page->view_info_button(
		url_ext => '/act,edit/id,' . $obj->{plugin}->id(),
		show_cancel   => 1,
		show_edit     => 1,
		show_delete   => 1,
		right_buttons => \@buttons
	);
}

sub edit{
	my $self = shift;
	my $plugin = $self->{obj}->{plugin};
	my $page = $self->{page};

	# cancel from view screen
	return sw_web_redirect_comprep(
		alias=>'payment_plugins_' . lc($plugin->sub_type),
	) if sw_argparam('item_cancel');

	if (sw_argparam('mark_active')) {
		my $err = $self->_mark_as_active(page=>$page);
		$page->add_status(string_id => "plug_active_ok") unless $err;
		$page->add_error(string    => $err) if $err;
		return $self->view();
	}
	
	if ( sw_argparam('item_delete') ) {
		$self->{obj}->{plugin}->delete();
		$page->{common_url} =
		  $ENV{COMPREP}->link( alias => $ENV{COMPREP}->node->{alias}, );
		return $page->redirect('op_ok');
	}

	## process "deactivate" action
	if (sw_argparam('deactivate_plugin')) {
		my $is_active = $plugin->is_active();

		if ($is_active) {
			$self->_activation_notify(is_active => 0);
			$plugin->is_active(0);
			$plugin->save();
			$page->add_status (string_id => 'plug_deactive_ok');
		}
		return $self->view();
	}
	return $self->edit_form();
}

sub edit_form {
	my $self = shift;
	
	my $page = $self->{page} or sw_die('page undefined');
	my $obj  = $self->{obj};
	
	$ENV{COMPREP}->node(edit => 1)->{title_hash}->{title} =
		$obj->{plugin}->name . " ". string('plugin');
	
	$page->title(
    	refresh_url	=> $page->get_browse_url(),
    	pages => [],
    );

	## check plugin is presence and defined
	unless ( defined($obj) && $obj->{plugin}->id ) {
		$page->add_error (
			string => string('plugin_unavail') . ' ' . string('try_again')
		);
		return $self->list();
	}

	my $form_url = $page->get_browse_url . "/act,update/id," . $obj->{plugin}->id();
	$page->edit_open(form_url => $form_url, is_multipart => 1);
	
	$page->edit_view_info(value => string('gen_settings'));
	
	$page->edit_view_input(
		title_id   => 'plugin_name_uc',
		view_name  => 'config_name',
		not_empty  => 1,
		obj_attr   => 'name',
		object     => $obj->{plugin},
		max_length => 50,
	);

	$page->edit_view_text(
		title_id   => 'plugin_template_uc',
		value      => HSPC::MT::PluginConfig->find_mt_template(
							template_id => $obj->{plugin}->template_id(),
							plugin_type => $obj->{plugin}->plugin_type,
							)->get_title(),
	);
	
	$page->edit_view_check(
		title_id  => 'active',
		view_name => 'is_active',
		value     => 1,
		is_checked=> $obj->{plugin}->is_active == 1 ? 1 : 0
	);

	if ( $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_PREAUTH) 
		&& $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_SALE) )
	{
		$page->edit_view_check(
			title_id  => 'preauth_enabled',
			view_name => 'is_preauth_enabled',
			value     => $obj->{plugin}->is_preauth_enabled,
			enable_views => $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_CAPTURE) ? 
					['start_billing_trans'] : [],
		);
	}

	if ( $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_PREAUTH) 
		&& $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_CAPTURE) )
	{
		$page->edit_view_combo(
			title_id  => 'start_billing_trans_uc',
			view_name => 'start_billing_trans',
			options   => [
					[ &SW_CCP_TRANS_TYPE_PREAUTH, 'preauth' ],
					[ &SW_CCP_TRANS_TYPE_CAPTURE, 'tt_capture' ]
			],
			not_empty => 1,
			value     => $obj->{plugin}->start_billing_trans,
			disabled  => (
				!$obj->{plugin}->is_preauth_enabled
				  && $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_SALE)
			),
		);
	}

	if ( scalar @{$obj->{plugin}->get_supported_payment_method_types} > 1 ) {
		$page->edit_view_lists2(
			title_data    => string('supported_cardtypes'),
			title_options => string('avail_cardtypes'),
			view_name     => 'card_types',
			options       => $obj->{plugin}->get_supported_payment_method_types,
			options_plain => 1,
			data          => [ map {  $_->{type}  } @{$obj->{plugin}->paymethod_types} ],
			ordered       => 0,
			not_empty     => 1,
			cols          => 2,
		);
	}

	$page->edit_view_info(value => string('plugin_setting_uc'));
	my $pr_template = $obj->{plugin}->find_presentation_template();
	my $content;
	my $is_provider = $ENV{security_obj}->account_type == &SW_HSP ? 1 : 0;

	eval {
		$content = $pr_template->edit_form(
						config      => $obj->{plugin}->config_data(),
						is_provider => $is_provider
					);
	};
	if ($@) {
		sw_log_warn("Error load presentation form: " . $@);
	}

	$page->edit_table_break(
		new_table_class => 'formborder',
		cellpadding => 5,
		height => 0,
		cell_open => 0,
	);

	$page->edit_append( content => $content );
	$page->edit_append( content => "<table>" );

	$page->edit_view_info(value => string('misc_cfg'));

	$page->edit_view_textarea(
		title_id => 'quick_descr_of_payment',
		obj_attr => 'banner_text',
		object   => $obj->{plugin},
		rows     => 5
	);

	$page->edit_close;
}

sub update {
	my $self = shift;
	my $obj  = $self->{obj};

	##my $templte_id  = sw_argparam('template_id');
	my $config_name = sw_argparam('name');
	my $is_active   = sw_argparam('is_active');

	return sw_web_redirect_comprep(
		alias=>'payment_plugins_'.lc($obj->{plugin}->sub_type()),
		[
			id => sw_uriparam('id'),
			act => 'view'
		]
	) if sw_argparam('item_cancel');

	##$obj->{plugin}->template_id($templte_id);
	$obj->{plugin}->name($config_name);

	$obj->{plugin}->banner_text(sw_argparam('banner_text'));
	$obj->{plugin}->is_preauth_enabled(sw_argparam('is_preauth_enabled'));
	$obj->{plugin}->start_billing_trans(sw_argparam('start_billing_trans') || &SW_CCP_TRANS_TYPE_PREAUTH );

	my $cc_types = sw_argparam('card_types','list');

	if ( scalar @{$cc_types}  > 0 ) {
		$obj->{plugin}->paymethod_types( $cc_types );
	}

	my $pr_template = $obj->{plugin}->find_presentation_template();

	my ($error, $config);
	my $is_provider = $ENV{security_obj}->account_type == &SW_HSP ? 1 : 0;

	try {
		my $pr_template = $obj->{plugin}->find_presentation_template();
		$config = $pr_template->collect_data(
			config      => $obj->{plugin}->config_data(),
			is_provider => $is_provider
		);
	} catch HSPC::Exception::Plugin::GeneralError with {
		my $e = shift;
		$error = $e->log_message;
		sw_log_warn($error);
		$error = $e->vendor_message 
			|| $e->user_message;
	} catch HSPC::Exception::Plugin::InvalidParameter with {
		my $e = shift;
		$error = $e->message;
		sw_log_warn($error);
	} otherwise {
		$error = shift;
	};

	if (!$error && $is_active) {
		$error = $self->_can_activate_plugin();
	}

	if (!$error) {
		$self->_activation_notify(is_active => $is_active);
		$obj->{plugin}->is_active($is_active);
		$obj->{plugin}->config_data($config);
		$error = $obj->{plugin}->save();
	}

	if($error) {
			$self->{page}->add_error(string => $error);
			return $self->edit_form();
	}

	return sw_web_redirect_comprep(
		alias=>'payment_plugins_' . lc($obj->{plugin}->sub_type),
		[
			id => $obj->{plugin}->id,
			act => 'view',
			sl => 'upd_ok_lc'
		]
	);
}

sub _activation_notify {
	my $self = shift;
	my %h    = (
		is_active => undef,
		@_
	);
	my $is_active = $h{is_active};
	my $plugin    = $self->{obj}->{plugin};

	if ( $plugin->is_active() != $is_active ) {
		my $template = $plugin->find_mt_template();
		if ($is_active) {
			if ( UNIVERSAL::can( $template, "plugin_activated" ) ) {
				eval { $template->plugin_activated( config => $plugin->config_data ); };
			}
		}
		else {
			if ( UNIVERSAL::can( $template, "plugin_deactivated" ) ) {
				eval { $template->plugin_deactivated( config => $plugin->config_data ); };
			}
		}
		if ($@) {
			sw_log_warn("Can't notify plugin error " . $@);
		}
	}
}


sub _can_activate_plugin {
	my $self = shift;

	my $plugin = $self->{obj};
	my $vendor_id = $ENV{security_obj}->account_no;

	my $pr_config = HSPC::MT::GM->find_provider_config(id => $vendor_id);
	if ((!$plugin->{plugin}->is_direct)
		 && ($pr_config->card_proc_type == &SW_GM_CCARD_PRTYPE_MANUAL)) {
		my $err = string(
			'plugin_cantbe_used_while_manual_approval_on',
			title => $plugin->get_title()
		);
		return $err;
	}

	unless ( $plugin->is_currency_supported(code => $pr_config->def_currency_code) ) {
		my $err = string('plugins_doesnt_support_currency',
					  currency  => $pr_config->def_currency_code,
					  plugins   => $plugin->{plugin}->name );
		return $err;
	}
	return undef;
}

sub _mark_as_active {
	my $self = shift;
	my $plugin = $self->{obj}->{plugin};

	## check plugin is presence and defined
	unless ( defined($plugin) && $plugin->id ) {
		return string('plugin_unavail') . ' ' . string('try_again');
	}

	my $err = $self->_can_activate_plugin();
	return $err if $err;
	
	$self->_activation_notify(is_active => 1);
	$plugin->is_active(1);
	my $err = $plugin->save;

	return $err;
}

sub get_help_url {
	my $class = shift;
	return HSPC::PP::PluginEngine->get_help_url();
}
1;
