package HSPC::PluginWizard::Abstract;

use strict;

use SWsoft::protect;
use HSPC::WebSystemLib;
use HSPC::Logger qw(sw_die sw_log sw_log_warn);
use HSPC::MT::PluginTools qw(set_request_data);
use HSPC::MT::Core::Constants qw(SW_HSP);
use Error qw(:try);
use HSPC::WebDB qw(atrans);

use vars qw(@ISA);
@ISA = qw(HSPC::UIStepsPage);

sub edit_teaser {
	my $class = shift;
	my %h = (
		page           => undef,
		created_object => undef,
		step           => undef,
		@_
	);
	my $self;

	if( $h{start} ) {
		$self = $class->new(start_step => $h{'step'} );

		if ( !$self->{obj} ) {
			$self->{obj} = $h{created_object};
		}
	} else {
		$self = 0;
	}

	my $res = $class->teaser_run(
				session_code => 'HSPC_PLUGIN',
				page         => $h{page},
				steps        => $self,
		);

	return $res;
}

sub process_url
{
	my $self = shift;

	if ($self->{obj}) {
		set_request_data('plugin_vendor_id', $self->{obj}->vendor_id() );
	}
	return $self->page->get_browse_url.'/act,wizard';
}

sub init_self_steps {
	my $self = shift;
	$self->add_step($self->{wz_steps});
	return $self->SUPER::init_self_steps(@_);
}

sub teaser_step_gen {
	my $self = shift;
	
	my $page = $self->page();
	$page->title(pages => []);
	$self->form_open();
	
	$page->edit_view_input(
		title_id   => 'plugin_name_uc',
		view_name  => 'config_name',
		not_empty  => 1,
		obj_attr   => 'name',
		object     => $self->{obj},
		max_length => 55,
	);

	my $templates = HSPC::MT::PluginConfig->find_templates(
				plugin_type => $self->{obj}->plugin_type,
				vendor_id   => $self->{obj}->vendor_id,
			);

	$page->edit_view_combo(
		title_id  => 'plugin_template_uc',
		view_name => 'template_id',
		options   => [map{[$_,HSPC::MT::PluginConfig->find_mt_template(
							template_id => $_,
							plugin_type => $self->{obj}->plugin_type,
							)->get_title()]}@{$templates}],
		not_empty     => 1,
		options_plain => 1,
		value         => 0,
	);

##	$page->edit_view_check(
##		title_id  =>'active',
##		view_name =>'is_active',
##		value     => 1,
##		is_checked=> $self->{obj}->is_active == 1 ? 1 : 0
##	);

	$self->form_close(edit_btns=>'save',new_btns=>'back_next');

	return {STATUS => 'OK'};
}

sub process_step_gen{
	my $self = shift;
	
	my $templte_id  = sw_argparam('template_id');
	my $config_name = sw_argparam('name');
	my $is_active   = sw_argparam('is_active');

	$self->{obj}->template_id($templte_id);
	$self->{obj}->name($config_name);
##	$self->{obj}->is_active($is_active);

	return {STATUS=>'PROCESS'};
}

sub handle_finish {
	my $self = shift;

	my $trans = atrans();
	$self->{obj}->save();

	set_request_data('plugin_config_id', $self->{obj}->id() );
	set_request_data('plugin_vendor_id', $self->{obj}->vendor_id() );
	set_request_data('plugin_template_id', $self->{obj}->template_id() );

	my ($error, $config);
	my $is_provider = $ENV{security_obj}->account_type == &SW_HSP ? 1 : 0;

	try {
		my $pr_template = $self->{obj}->find_presentation_template();
		$config = $pr_template->collect_data(
			config      => $self->{obj}->config_data(),
			is_provider => $is_provider
		);
	} catch HSPC::Exception::Plugin::GeneralError with {
		my $e = shift;
		$error = $e->log_message;
		sw_log_warn($error);
		$error = $e->vendor_message;
	} catch HSPC::Exception::Plugin::InvalidParameter with {
		my $e = shift;
		$error = $e->message;
	} otherwise {
		$error = shift;
	};
	
	if ($error) {
		$self->{obj}->id(0);
		$trans->rollback();
		$self->add_error( string => "$error" );
		return {step_inc => 0};
	}

	$self->{obj}->config_data($config);

	$self->{obj}->save();
	$trans->commit();

	$self->{free_session} = 1;

	return $self->redirect(sl => 'configuration_added_ok');
}

sub handle_cancel {
	my $self = shift;
	$self->{free_session} = 1;

	$self->redirect();
}
1;
