package HSPC::PluginWizard::PP;

use SWsoft::protect;
use strict;
use HSPC::WebSystemLib;
use HSPC::PluginWizard::Abstract;
use Data::Dumper;
use HSPC::Logger qw(sw_die sw_log sw_log_warn);
use HSPC::Localization;
use HSPC::MT::PluginTools qw(set_request_data);
use HSPC::MT::CCP::Constants qw(:trans_types);
use HSPC::MT::Core::Constants qw(SW_HSP);
use HSPC::MT::PluginConfig::PP;

use vars qw(@ISA);
@ISA = qw(HSPC::PluginWizard::Abstract);

my @STEPS = (
		{title_id => 'general_info_uc',	code => 'gen'},
		{title_id => 'plugin_conf_uc',	code => 'plugin'},
);

sub new {
	my $class = shift;

	my $obj = $class->SUPER::new(@_);

	if ($obj) {
		push @{$obj->{wz_steps}}, @STEPS;
	}
	return $obj;
}

sub edit_teaser {
	my ($class,%h) = @_;
	my $class = shift;
	my %h = (
		page     => undef,
		sub_type => undef,
		start    => undef,
		@_
	);

	if ($h{start}) {
		my $obj = HSPC::MT::PluginConfig::PP->new();
		$obj->plugin_type("PP");
		$obj->sub_type( $h{sub_type} );
		$obj->vendor_id( $ENV{security_obj}->account_no() );
		$h{created_object} = $obj;
	}
	$class->SUPER::edit_teaser(%h);
}

sub teaser_step_gen {
	my $self = shift;
	
	my $page = $self->page();
	$page->title(pages => []);
	$self->form_open();
	
	$page->edit_view_input(
		title_id   => 'plugin_name_uc',
		view_name  => 'config_name',
		not_empty  => 1,
		obj_attr   => 'name',
		object     => $self->{obj},
		max_length => 55,
	);

	my $templates = HSPC::MT::PluginConfig::PP->find_templates(
		plugin_type => $self->{obj}->plugin_type,
		vendor_id   => $self->{obj}->vendor_id,
		sub_type    => $self->{obj}->sub_type,
	);

	$page->edit_view_combo(
		title_id  => 'plugin_template_uc',
		view_name => 'template_id',
		options   => [
			sort {uc($a->[1]) cmp uc($b->[1])}
				map{[$_,HSPC::MT::PluginConfig::PP->find_mt_template(
							template_id => $_,
							plugin_type => $self->{obj}->plugin_type,
							)->get_title()]}@{$templates}],
		not_empty     => 1,
		options_plain => 1,
		value         => 0,
	);

##	$page->edit_view_check(
##		title_id  =>'active',
##		view_name =>'is_active',
##		value     => 1,
##		is_checked=> $self->{obj}->is_active == 1 ? 1 : 0
##	);

	$self->form_close(edit_btns=>'save',new_btns=>'back_next');

	return {STATUS => 'OK'};
}

sub teaser_step_plugin {
	my $self = shift;
	my $page = $self->page();

	my $plugin_conf  = $self->{obj};
	my $obj = HSPC::MT::PP::PluginEngine->find( plugin => $plugin_conf );

	set_request_data('plugin_template_id', $obj->{plugin}->template_id() );

	$page->title( pages => [] );

	$self->form_open( is_multipart => 1);

	if ( $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_PREAUTH) 
		&& $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_SALE) )
	{
		$page->edit_view_check(
			title_id  => 'preauth_enabled',
			view_name => 'is_preauth_enabled',
			value     => $obj->{plugin}->is_preauth_enabled,
			enable_views => $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_CAPTURE) ? 
					['start_billing_trans'] : [],
		);
	}

	if ( $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_PREAUTH) 
		&& $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_CAPTURE) )
	{
		$page->edit_view_combo(
			title_id  => 'start_billing_trans_uc',
			view_name => 'start_billing_trans',
			options   => [
					[ &SW_CCP_TRANS_TYPE_PREAUTH, 'preauth' ],
					[ &SW_CCP_TRANS_TYPE_CAPTURE, 'tt_capture' ]
			],
			not_empty => 1,
			value     => $obj->{plugin}->start_billing_trans,
			disabled  => (
				!$obj->{plugin}->is_preauth_enabled
				  && $obj->is_trans_type_supported(&SW_CCP_TRANS_TYPE_SALE)
			),
		);
	}

	if ( scalar @{ $obj->{plugin}->get_supported_payment_method_types} > 1 ) {
		$page->edit_view_info(value => string('gen_settings'));
		$page->edit_view_lists2(
			title_data    => string('supported_cardtypes'),
			title_options => string('avail_cardtypes'),
			view_name     => 'card_types',
			options       => $obj->{plugin}->get_supported_payment_method_types,
			options_plain => 1,
			ordered       => 0,
			not_empty     => 1,
			cols          => 2,
		);
	}

	my $pr_template = $self->{obj}->find_presentation_template();
	my $is_provider = $ENV{security_obj}->account_type == &SW_HSP ? 1 : 0;

	my $content;
	eval {
		$content = $pr_template->edit_form(
						config      => $self->{obj}->config_data(),
						is_provider => $is_provider
					);
	};
	if ($@) {
		sw_log_warn("Error load presentation form: " . $@);
	}

	$page->edit_view_info(value => string('plugin_setting_uc'));

	$page->edit_table_break(
		new_table_class => 'formborder',
		cellpadding => 5,
		height => 0,
		cell_open => 0,
	);

	$page->edit_append( content => $content );
	$page->edit_append( content => "<table>" );

	$page->edit_view_info(value => string('misc_cfg'));

	$page->edit_view_textarea(
		title_id => 'quick_descr_of_payment',
		obj_attr => 'banner_text',
		object   => $obj->{plugin},
		rows     => 5
	);

	$self->form_close(new_btns=>'cancel_back_finish');

	return {STATUS => 'OK'};
}

sub process_step_plugin {
	my $self = shift;
	
	my $obj  = $self->{obj};

	my $pr_template = $self->{obj}->find_presentation_template();
	my $config;

	my $cc_types = sw_argparam('card_types','list');
	push(@$cc_types, map { $_->[0] } @{$self->{obj}->get_supported_payment_method_types})
		unless @$cc_types; 

	$self->{obj}->paymethod_types($cc_types);
	$self->{obj}->is_preauth_enabled( sw_argparam('is_preauth_enabled') )
	  if sw_argparam('is_preauth_enabled');
	$self->{obj}->start_billing_trans(sw_argparam('start_billing_trans') || &SW_CCP_TRANS_TYPE_PREAUTH );

	$obj->banner_text(sw_argparam('banner_text'));
	
	return {STATUS => 'PROCESS'};
}

sub redirect {
	my ($self,%h) = @_;

	return sw_web_redirect_comprep(alias => 'payment_plugins_' . lc($self->{obj}->sub_type) );
}

1;
